import json
from typing import Optional, List

from .color import Color
from .modules import _Module

__all__ = ['Card']


class Card:
    """
    构建卡片
    """
    type: str = 'card'
    theme: str
    size: str
    color: [Color, str]
    modules: List[_Module]

    def __init__(self, modules: List[_Module], theme: str = 'primary', size: str = 'lg', color: [Color, str] = None) -> None:
        """
        构建卡片

        :param modules: 卡片模块列表
        :param theme: 卡片主题
        :param size: 目前只支持sm与lg。 lg仅在PC端有效, 在移动端不管填什么，均为sm。
        :param color: 卡片颜色 ex: #55ffff or NamedColor.XXX
        """
        self.modules = modules
        self.theme = theme
        self.size = size
        self.color = None
        if isinstance(color, Color):
            self.color = color.__str__()
        if isinstance(color, str):
            self.color = color

    def __getitem__(self, item: int):
        return self.modules[item]

    def build(self) -> dict:
        """
        :return: 构造后卡片
        """
        ret = {'type': self.type, 'theme': self.theme, 'size': self.size, 'modules': []}
        if self.color is not None:
            ret['color'] = self.color
        for i in self.modules:
            ret['modules'].append(i.build())
        return ret

    def build_to_json(self) -> str:
        return json.dumps(self.build(), indent=4, ensure_ascii=False)

    def append(self, module: _Module):
        self.modules.append(module)

    def clear(self):
        self.modules.clear()



from .color import Color

__all__ = ['Theme', 'Size', 'NamedColor']


class Theme:
    PRIMARY: str = 'primary'
    SUCCESS: str = 'success'
    DANGER: str = 'danger'
    WARNING: str = 'warning'
    INFO: str = 'info'
    SECONDARY: str = 'secondary'
    NONE: str = 'none'


class Size:
    XS: str = 'xs'
    SM: str = 'sm'
    MD: str = 'md'
    LG: str = 'lg'


class NamedColor:
    BLACK: Color = Color(0, 0, 0)
    DARK_BLUE: Color = Color(0, 0, 170)
    DARK_GREEN: Color = Color(0, 170, 0)
    DARK_AQUA: Color = Color(0, 170, 170)
    DARK_RED: Color = Color(170, 0, 0)
    DARK_PURPLE: Color = Color(170, 0, 170)
    GOLD: Color = Color(255, 170, 0)
    GRAY: Color = Color(170, 170, 170)
    DARK_GRAY: Color = Color(85, 85, 85)
    BLUE: Color = Color(85, 85, 255)
    GREEN: Color = Color(85, 255, 85)
    AQUA: Color = Color(85, 255, 255)
    RED: Color = Color(255, 85, 85)
    YELLOW: Color = Color(255, 255, 85)
    WHITE: Color = Color(255, 255, 255)



class Color:
    """
    添加颜色
    """
    R: int
    G: int
    B: int

    def __init__(self, r: int, g: int, b: int) -> None:
        """
        添加颜色

        :param r: 红
        :param g: 绿
        :param b: 蓝
        """
        if r > 255 or g > 255 or b > 255:
            raise Exception('RGB数字必须为0-255')
        if r < 0 or g < 0 or b < 0:
            raise Exception('RGB数字必须为0-255')
        self.R = r
        self.G = g
        self.B = b

    def __str__(self) -> str:
        r = str(hex(self.R))[2:]
        g = str(hex(self.G))[2:]
        b = str(hex(self.B))[2:]
        if len(r) == 1:
            r = '0' + r
        if len(g) == 1:
            g = '0' + g
        if len(b) == 1:
            b = '0' + b
        return f'#{r+g+b}'
